;;;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp; Base: 10; Package: modlisp -*-
;;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          base.lisp
;;;; Purpose:       Utility functions for modlisp package
;;;; Programmer:    Kevin M. Rosenberg
;;;; Date Started:  Dec 2002
;;;;
;;;; $Id: base.lisp,v 1.6 2003/07/08 06:40:00 kevin Exp $
;;;; *************************************************************************

(in-package #:modlisp)

(defun modlisp-start (&key (port +default-apache-port+)
			   (processor 'demo-apache-command-processor)
			   (processor-args nil))
  (let ((listener (make-instance 'listener :port port
				 :name (next-server-name)
				 :function 'apache-command-issuer
				 :function-args (cons processor processor-args)
				 :format :text
				 :wait nil)))
    (handler-case
	(make-socket-server listener)
      (error (e)
	(format t "~&Error while trying to start modlisp server~&  ~A" e)
	(decf *listener-count*)
	(values nil nil))
      (:no-error (res)
	(declare (ignore res))
	(push listener *active-listeners*)
	listener))))
    
(defun modlisp-stop (listener)
  (unless listener
    (cmsg "listener is NIL in modlisp-stop")
    (return-from modlisp-stop))
  (dolist (worker (workers listener))
    (close-active-socket (connection worker))
    (destroy-process (process worker)))
  (setf (workers listener) nil)
  (with-slots (process socket) listener
    (errorset (close-passive-socket socket) t)
    (errorset (destroy-process process) t))
  (setq *active-listeners* (remove listener *active-listeners*)))

(defun modlisp-stop-all ()
  (dolist (listener *active-listeners*)
    (ignore-errors
     (progn
       (modlisp-stop listener)
       (setq *active-listeners* (remove listener *active-listeners*))))))

(defun next-server-name ()
  (format nil "modlisp-socket-server-~d" (incf *listener-count*))) 

(defun next-worker-name ()
  (format nil "modlisp-worker-~d" (incf *worker-count*)))

(let ((*number-server-requests* 0)
      (*number-worker-requests* 0)
      (*close-apache-socket* t))
  
  (defun apache-command-issuer (*apache-socket* processor &rest args)
    "generates commands from apache, issues commands to processor-fun"
    (unwind-protect
	 (progn
	   (setq *number-worker-requests* 0)
	   (do ((command (read-apache-command) (read-apache-command)))
	       ((null command))
	     (apply processor command args)
	     (force-output *apache-socket*)
	     (incf *number-worker-requests*)
	     (incf *number-server-requests*)
	     (when *close-apache-socket*
	       (return))))
      (close-active-socket *apache-socket*)))
  

  (defun get-number-worker-requests ()
    *number-worker-requests*)

  (defun get-number-server-requests ()
    *number-server-requests*)
  
  (defun set-close-apache-socket (close?)
    (setq *close-apache-socket* close?))
  
  ) ;; closure

(defun header-value (header key)
  (cdr (assoc key header :test #'string=)))

(defun read-apache-command ()
  (ignore-errors
    (let* ((header (read-apache-header))
	   (content-length (header-value header "content-length"))
	   (content (when content-length 
		      (make-string
		       (parse-integer content-length :junk-allowed t)))))
     (when content
       (read-sequence content *apache-socket*)
       (push (cons "posted-content" content) header))
     header)))

(defun read-apache-line ()
  (kmrcl:string-left-trim-one-char  #\return
				    (read-line *apache-socket* nil nil)))

(defun read-apache-header ()
  (loop for key = (read-apache-line)
      while (and key
		 (string-not-equal key "end")
		 (> (length key) 1))
      for value = (read-apache-line)
      collect (cons key value)))

(defun write-header-line (key value)
  (write-string key *apache-socket*)
  (write-char #\NewLine *apache-socket*)
  (write-string value *apache-socket*)
  (write-char #\NewLine *apache-socket*))


